import React, { useCallback } from "react";
import Editor from "@monaco-editor/react";

const activateMonacoJSXHighlighter = async (monacoEditor: any, monaco: any) => {
  // monaco-jsx-highlighter depends on these in addition to Monaco and an instance of a Monaco Editor.
  const { default: traverse } = await import("@babel/traverse");
  const { parse } = await import("@babel/parser");
  // >>> The star of the show =P >>>
  const {
    default: MonacoJSXHighlighter,
    JSXTypes,
    makeBabelParse, //By @HaimCandiTech
  } = await import(
    // @ts-ignore
    "monaco-jsx-highlighter" // Note: there is a polyfilled version alongside the regular version.
  ); // For example, starting with 2.0.2, 2.0.2-polyfilled is also available.

  const parseJSX = makeBabelParse(parse, true); // param0:Babel's parse, param1: default config for JSX syntax (false), TSX (true).
  // Instantiate the highlighter
  const monacoJSXHighlighter = new MonacoJSXHighlighter(
    monaco, // references Range and other APIs
    parseJSX, // obtains an AST, internally passes to parse options: {...options, sourceType: "module",plugins: ["jsx"],errorRecovery: true}
    traverse, // helps collecting the JSX expressions within the AST
    monacoEditor, // highlights the content of that editor via decorations
  );
  // Start the JSX highlighting and get the dispose function
  let disposeJSXHighlighting =
    monacoJSXHighlighter.highlightOnDidChangeModelContent();
  // Enhance monaco's editor.action.commentLine with JSX commenting and get its disposer
  let disposeJSXCommenting = monacoJSXHighlighter.addJSXCommentCommand();
  // <<< You are all set. >>>

  // Optional: customize the color font in JSX texts (style class JSXElement.JSXText.tastyPizza from ./index.css)
  JSXTypes.JSXText.options.inlineClassName = "JSXElement.JSXText.tastyPizza";
  // more details here: https://microsoft.github.io/monaco-editor/api/interfaces/monaco.editor.IModelDecorationOptions.html
  // console.log(
  //   "Customize each JSX expression type's options, they must match monaco.editor.IModelDecorationOptions:",
  //   JSXTypes,
  // );

  // This example's shorthands for toggling actions
  const toggleJSXHighlighting = () => {
    if (disposeJSXHighlighting) {
      disposeJSXHighlighting();
      disposeJSXHighlighting = null;
      return false;
    }

    disposeJSXHighlighting =
      monacoJSXHighlighter.highlightOnDidChangeModelContent();
    return true;
  };

  const toggleJSXCommenting = () => {
    if (disposeJSXCommenting) {
      disposeJSXCommenting();
      disposeJSXCommenting = null;
      return false;
    }

    disposeJSXCommenting = monacoJSXHighlighter.addJSXCommentCommand();
    return true;
  };

  const isToggleJSXHighlightingOn = () => !!disposeJSXHighlighting;
  const isToggleJSXCommentingOn = () => !!disposeJSXCommenting;

  return {
    monacoJSXHighlighter,
    toggleJSXHighlighting,
    toggleJSXCommenting,
    isToggleJSXHighlightingOn,
    isToggleJSXCommentingOn,
  };
};

type CodeEditorProps = {
  // code: string;
  defaultCode?: string;
  onChange: (value: string | undefined) => void;
  theme: string;
  language: string;
  codeValue: string;
};
export function CodeEditor({
  defaultCode,
  onChange,
  theme,
  language,
  codeValue,
}: CodeEditorProps) {
  // const [value, setValue] = useState("");

  const handleEditorDidMount = useCallback((monacoEditor: any, monaco: any) => {
    // monacoEditor.updateOptions({ wordWrap: "on" });
    activateMonacoJSXHighlighter(monacoEditor, monaco)
      .then((monacoJSXHighlighterRefCurrent) => {
        // monacoJSXHighlighterRef.current = monacoJSXHighlighterRefCurrent;
        // setIsEditorReady(!!monacoEditor);
        // setIsJSXHighlightingOn(
        //   monacoJSXHighlighterRefCurrent.isToggleJSXHighlightingOn()
        // );
        // setIsJSXCommentingOn(
        //   monacoJSXHighlighterRefCurrent.isToggleJSXCommentingOn()
        // );
      })
      .catch((e) => {
        // console.log(e)
      });
  }, []);

  // const handleEditorChange = (value: string | undefined) => {
  //   setValue(value || "");
  //   onChange(value || "");
  // };

  return (
    <Editor
      // height="50vh" // By default, it fully fits with its parent
      options={{
        minimap: {
          enabled: false,
        },
        wordWrap: "on"
      }}
      theme={theme}
      language={language === "react" ? "javascript" : language}
      value={codeValue}
      defaultValue={defaultCode}
      onMount={handleEditorDidMount}
      onChange={onChange}

    />
  );
}
